/* Licensed to Stichting The Commons Conservancy (TCC) under one or more
 * contributor license agreements.  See the AUTHORS file distributed with
 * this work for additional information regarding copyright ownership.
 * TCC licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#define WM_MESSAGE_RECEIVE WM_APP+1
#define WM_MESSAGE_RECEIVED WM_APP+2
#define WM_MESSAGE_SEND WM_APP+3
#define WM_MESSAGE_SENT WM_APP+4
#define WM_MESSAGE_SHOW WM_APP+5

#define WM_CRYPTO_UPDATED WM_APP+101
#define WM_CRYPTO_SELECTED WM_APP+102
#define WM_CRYPTO_SIGNED WM_APP+103
#define WM_CRYPTO_NOTSIGNED WM_APP+104

#define IDT_TIMER_CERTS WM_APP+201

#include <vector>
#include <memory>

#include "SignPage.h"

// CRedwaxSignTextMFCDlg dialog
class CRedwaxSignTextMFCDlg : public CDialogEx
{
// Construction
public:
	CRedwaxSignTextMFCDlg(CWnd* pParent = nullptr);	// standard constructor
	~CRedwaxSignTextMFCDlg();	// standard destructor

// Dialog Data
#ifdef AFX_DESIGN_TIME
	enum { IDD = IDD_REDWAX_SIGNTEXT_MFC_DIALOG };
#endif

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:

	HANDLE  hThread;
	HANDLE  cThread;
	SignTextData* signtext;

	HICON m_hIcon;
	
	CTabCtrl	m_ctlTab;

	std::vector<std::shared_ptr<SignPage>> m_dlgs;

	CButton m_sign;
	CButton m_cancel;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSelchangeTab1(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSelchangeCertBox(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg LRESULT OnMessageReceive(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMessageReceived(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMessageSend(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMessageSent(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnMessageShow(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnCryptoUpdated(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnCryptoSelected(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT CRedwaxSignTextMFCDlg::OnCryptoSigned(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT CRedwaxSignTextMFCDlg::OnCryptoNotSigned(WPARAM wParam, LPARAM lParam);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnSizing(UINT fwSide, LPRECT pRect);
	afx_msg void OnGetMinMaxInfo(MINMAXINFO* lpMMI);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	DECLARE_MESSAGE_MAP()

private:
	void ResizeTab();
	void UpdateVisibleTab();
public:
	afx_msg void OnClose();
	afx_msg void OnBnClickedSign();
	afx_msg void OnBnClickedCancel();
	afx_msg void OnIdok();
};
